/* -*- Mode: C; indent-tabs-mode:nil; c-basic-offset:8 -*- */
/*This file is part of GNU MlView
 *
 *GNU MlView is free software; you can redistribute 
 *it and/or modify it under the terms of 
 *the GNU General Public License as published by the 
 *Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *GNU MlView is distributed in the hope that it will be useful, 
 *but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of 
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the 
 *GNU General Public License along with MlView; 
 *see the file COPYING. 
 *If not, write to the Free Software Foundation, 
 *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 *Copyright 2001-2002 Dodji Seketeli <dodji@seketeli.org>
 */

#ifndef __MLVIEW_UTILS_H__
#define __MLVIEW_UTILS_H__

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlerror.h>
#include <gnome.h>
#include "mlview-app-context.h"

#ifdef __cpluplus
extern "C"{
#endif /*__cpluplus*/
	
/**
 *@file
 *Misc usefull functions.
 *There are also some functions that could be small patches
 *gtk and libxml2.
 */

	enum MlViewStatus {
		MLVIEW_OK,
		MLVIEW_BAD_PARAM_ERROR,
		MLVIEW_NULL_FILE_NAME_ERROR,
		MLVIEW_BAD_FILE_ERROR,
		MLVIEW_EMPTY_FILE_ERROR,
		MLVIEW_NO_SPACE_LEFT_ERROR,
		MLVIEW_UNKNOWN_ENCODING_ERROR,
		MLVIEW_ENCODING_ERROR,
		MLVIEW_NO_ENCODINGS_ERROR,
		MLVIEW_ENCODING_NOT_SUPPORTED_ERROR,
		MLVIEW_CHAR_TOO_LONG,
		/*must be the last one*/
		MLVIEW_ERROR
	} ;
  
	enum MlViewEncoding {
		UTF8,
		ISO8859_1,
		ASCII
	} ;

/*
 *Some miscellaneous macros definitions.
 */

#define MVLIEW_LOG_DOMAIN "MLVIEW"

#ifdef __GNUC__

#define mlview_utils_trace(a_log_level, a_msg) \
g_log (MVLIEW_LOG_DOMAIN, \
       G_LOG_LEVEL_CRITICAL, \
       "file %s: line %d (%s): %s\n", \
       __FILE__, \
       __LINE__, \
       __PRETTY_FUNCTION__, \
	a_msg)
#else /*__GNUC__*/

#define mlview_utils_trace(a_log_level, a_msg) \
g_log (MVLIEW_LOG_DOMAIN, \
       G_LOG_LEVEL_CRITICAL, \
       "file %s: line %d: %s\n", \
       __FILE__, \
       __LINE__, \
       a_msg)
#endif

/**
 *Traces an info message.
 *The file, line and enclosing function
 *of the message will be automatically 
 *added to the message.
 *@param a_msg the msg to trace.
 */
#define mlview_utils_trace_info(a_msg) \
mlview_utils_trace (G_LOG_LEVEL_INFO, a_msg)

/**
 *Trace a debug message.
 * The file, line and enclosing function
 *of the message will be automatically
 *added to the message.
 *@param a_msg the msg to trace.
 */
#define mlview_utils_trace_debug(a_msg) \
mlview_utils_trace (G_LOG_LEVEL_DEBUG, a_msg) ;

	int 
	mlview_utils_load_xml_document (gchar * a_xmlfile,
					xmlParserCtxtPtr a_parser_context) ;

	void 
	mlview_utils_get_xml_parser_error_as_string1
	(xmlParserCtxtPtr a_xml_parser_context, gchar * error_string) ;

        void
        mlview_utils_set_window_deco (GtkWindow * a_w,
                                      MlViewAppContext *a_ctxt,
                                      guchar * a_wm_class_name) ;

        void
        mlview_utils_display_error_dialog (MlViewAppContext * a_ctxt,
                                           gchar *a_msg_format, ...) ;

	void 
	mlview_utils_get_xml_parser_error_as_string2 
        (xmlParserErrors a_xml_error, gchar * error_string) ;

	gboolean 
	mlview_utils_is_white_string (const gchar *a_str) ;
	
	enum MlViewStatus
	mlview_utils_isolat1_str_len_as_utf8 (guchar *a_str,
					      gint *a_result_len) ;

	enum MlViewStatus
	mlview_utils_utf8_str_len_as_isolat1 (guchar *a_utf8_str,
					      gint *a_len) ;

	enum MlViewStatus
	mlview_utils_isolat1_str_to_utf8 (guchar *a_input_str,
					  guchar **a_output_str) ;
	
	enum MlViewStatus
	mlview_utils_utf8_str_to_isolat1 (guchar *a_in_str,
					  guchar **a_out_str) ;
	void 
	mlview_utils_parse_full_name (xmlNode *a_node, guchar *a_full_name, 
				      xmlNs ** a_ns, guchar **a_local_name) ;

	void
	mlview_utils_init_available_encodings_list (void) ;

	void
	mlview_utils_ref_available_encodings (void) ;

	void
	mlview_utils_unref_available_encodings (void) ;

	GList *
	mlview_utils_get_available_encodings (void) ;
	
	enum MlViewStatus
	mlview_utils_add_supported_encoding (guchar *a_name) ;

	enum MlViewStatus
	mlview_utils_del_supported_encoding (guchar *a_name) ;

	gboolean
	mlview_utils_is_encoding_supported (guchar *a_encoding) ;
		
	xmlNs * 
	xmlUnlinkNs (xmlNode * node, xmlNs * ns) ;

	xmlNs * 
	xmlUnlinkNsDef (xmlNode * node, xmlNs * ns) ;

	int
	xmlValidGetValidElementsChildren (xmlNode *a_node, 
					  const xmlChar **a_list,
					  int a_max) ;

	gint 
	gtk_clist_row_absolute_top_ypixel (GtkCList *a_clist, 
					   gint a_row) ;

	gint 
	gtk_ctree_node_absolute_top_ypixel (GtkCTree * a_tree, 
					    GtkCTreeNode *a_node) ;

	void 
	mlview_utils_init_i18n (void) ;

#ifdef __cpluplus
}
#endif //__cpluplus
#endif
